import sys
import os
import yaml
import appdirs
from packaging import version

# Location of the sasktranif registry yaml file
REGISTRY_FILE = os.path.join(sys.exec_prefix, 'share', 'usask-arg', 'registry', 'sasktranif', 'globalkey.yaml')

# Mapping from option in the user config file to sasktranif registry setting
OPTION_MAPPINGS = {'hitran_directory': ['software', 'usask-arg', 'skopticalproperties',
                                        'hitran', 'basedirectory'],
                   'baum_directory': ['software', 'usask-arg', 'skopticalproperties', 'baum_icecrystals', 'storage',
                                      '2014database'],
                   'ecmwf_directory': ['software', 'usask-arg', 'climatology', 'ecmwf', 'storage', 'erainterimdir']}


def load_sasktran_registry():
    """
    Opens the sasktranif registry as a dictionary
    """
    if version.parse(yaml.__version__) > version.parse('5'):
        return yaml.load(open(REGISTRY_FILE), Loader=yaml.FullLoader)
    else:
        return yaml.load(open(REGISTRY_FILE))


def user_config_file_location():
    """
    File location of the user config file
    """
    user_config_dir = appdirs.user_config_dir('sasktran')

    user_config_file = os.path.join(user_config_dir, 'config.yml')

    return user_config_file


def load_user_config():
    """
    Opens the user config file as a dictionary
    """
    user_config_file = user_config_file_location()

    if version.parse(yaml.__version__) > version.parse('5'):
        return yaml.load(open(user_config_file), Loader=yaml.FullLoader)
    else:
        return yaml.load(open(user_config_file))


def merged_registry(user_config, sasktran_registry):
    """
    Merges the user config file into the sasktran registry dictionary.  Returns a dictionary of merged values.
    """
    for key, user_val in user_config.items():
        try:
            mapping = OPTION_MAPPINGS[key]
        except KeyError:
            # Either typo or an option that does not go into the registry
            continue

        # Traverse the registry except for the last entry
        base = sasktran_registry
        for m in mapping[:-1]:
            base = base[m]
        # Assign the last entry to the user value
        base[mapping[-1]] = user_val

    return sasktran_registry


def save_registry(registry):
    """
    Rewrites the sasktranif registry according to an input dictionary
    """
    yaml.dump(registry, open(REGISTRY_FILE, 'w'), default_flow_style=False)


def update_registry_from_config():
    """
    Rewrites the sasktranif registry using the user config file to merge in new values.
    """
    try:
        user_config = load_user_config()
    except FileNotFoundError:
        # No config file, we dont have to do anything
        return
    sasktran_reg = load_sasktran_registry()
    merged = merged_registry(user_config, sasktran_reg)
    save_registry(merged)
